@@ -1,5 +1,6 @@
 from parcels.tools.loggers import logger
 from parcels.tools.converters import unitconverters_map, UnitConverter, Geographic, GeographicPolar
+from parcels.tools.converters import TimeConverter
 from parcels.tools.error import FieldSamplingError, TimeExtrapolationError
 from collections import Iterable
 from py import path
@@ -44,7 +45,7 @@ class Field(object):
     :param transpose: Transpose data to required (lon, lat) layout
     :param vmin: Minimum allowed value on the field. Data below this value are set to zero
     :param vmax: Maximum allowed value on the field. Data above this value are set to zero
-    :param time_origin: Time origin (datetime or np.datetime64 object) of the time axis (only if grid is None)
+    :param time_origin: Time origin (TimeConverter object) of the time axis (only if grid is None)
     :param interp_method: Method for interpolation. Either 'linear' or 'nearest'
     :param allow_time_extrapolation: boolean whether to allow for extrapolation in time
            (i.e. beyond the last available time snapshot)
@@ -57,6 +58,7 @@ class Field(object):
                  interp_method='linear', allow_time_extrapolation=None, time_periodic=False, **kwargs):
         self.name = name
         self.data = data
+        time_origin = TimeConverter(0) if time_origin is None else time_origin
         if grid:
             self.grid = grid
         else:
@@ -68,7 +70,6 @@ class Field(object):
         self.lon = self.grid.lon
         self.lat = self.grid.lat
         self.depth = self.grid.depth
-        self.time = self.grid.time
         fieldtype = self.name if fieldtype is None else fieldtype
         if self.grid.mesh == 'flat' or (fieldtype not in unitconverters_map.keys()):
             self.units = UnitConverter()
@@ -118,16 +119,20 @@ class Field(object):
         self.dimensions = kwargs.pop('dimensions', None)
         self.indices = kwargs.pop('indices', None)
         self.dataFiles = kwargs.pop('dataFiles', None)
+        self.netcdf_engine = kwargs.pop('netcdf_engine', 'netcdf4')
         self.loaded_time_indices = []
 
     @classmethod
     def from_netcdf(cls, filenames, variable, dimensions, indices=None, grid=None,
                     mesh='spherical', allow_time_extrapolation=None, time_periodic=False,
-                    full_load=False, dimension_filename=None, **kwargs):
+                    full_load=False, **kwargs):
         """Create field from netCDF file
 
         :param filenames: list of filenames to read for the field.
                Note that wildcards ('*') are also allowed
+               filenames can be a list [files]
+               or a dictionary {dim:[files]} (if lon, lat, depth and/or data not stored in same files as data)
+               time values are in filenames[data]
         :param variable: Name of the field to create. Note that this has to be a string
         :param dimensions: Dictionary mapping variable names for the relevant dimensions in the NetCDF file
         :param indices: dictionary mapping indices for each dimension to read from file.
@@ -147,21 +152,47 @@ class Field(object):
                It is advised not to fully load the data, since in that case Parcels deals with
                a better memory management during particle set execution.
                full_load is however sometimes necessary for plotting the fields.
+        :param netcdf_engine: engine to use for netcdf reading in xarray. Default is 'netcdf',
+               but in cases where this doesn't work, setting netcdf_engine='scipy' could help
         """
 
         if not isinstance(filenames, Iterable) or isinstance(filenames, str):
             filenames = [filenames]
-        dimension_filename = dimension_filename if dimension_filename else filenames[0]
+
+        data_filenames = filenames['data'] if type(filenames) is dict else filenames
+        if type(filenames) == dict:
+            for k in filenames.keys():
+                assert k in ['lon', 'lat', 'depth', 'data'], \
+                    'filename dimension keys must be lon, lat, depth or data'
+            assert len(filenames['lon']) == 1
+            if filenames['lon'] != filenames['lat']:
+                raise NotImplementedError('longitude and latitude dimensions are currently processed together from one single file')
+            lonlat_filename = filenames['lon'][0]
+            if 'depth' in dimensions:
+                depth_filename = filenames['depth'][0]
+                if len(depth_filename) != 1:
+                    raise NotImplementedError('Vertically adaptive meshes not implemented for from_netcdf()')
+        else:
+            lonlat_filename = filenames[0]
+            depth_filename = filenames[0]
+
         indices = {} if indices is None else indices.copy()
-        with NetcdfFileBuffer(dimension_filename, dimensions, indices) as filebuffer:
+        netcdf_engine = kwargs.pop('netcdf_engine', 'netcdf4')
+        with NetcdfFileBuffer(lonlat_filename, dimensions, indices, netcdf_engine) as filebuffer:
             lon, lat = filebuffer.read_lonlat
-            depth = filebuffer.read_depth
             indices = filebuffer.indices
             # Check if parcels_mesh has been explicitly set in file
             if 'parcels_mesh' in filebuffer.dataset.attrs:
                 mesh = filebuffer.dataset.attrs['parcels_mesh']
 
-        if len(filenames) > 1 and 'time' not in dimensions:
+        if 'depth' in dimensions:
+            with NetcdfFileBuffer(depth_filename, dimensions, indices, netcdf_engine) as filebuffer:
+                depth = filebuffer.read_depth
+        else:
+            indices['depth'] = [0]
+            depth = np.zeros(1)
+
+        if len(data_filenames) > 1 and 'time' not in dimensions:
             raise RuntimeError('Multiple files given but no time dimension specified')
 
         if grid is None:
@@ -169,23 +200,20 @@ class Field(object):
             # across multiple files
             timeslices = []
             dataFiles = []
-            for fname in filenames:
-                with NetcdfFileBuffer(fname, dimensions, indices) as filebuffer:
+            for fname in data_filenames:
+                with NetcdfFileBuffer(fname, dimensions, indices, netcdf_engine) as filebuffer:
                     ftime = filebuffer.time
                     timeslices.append(ftime)
                     dataFiles.append([fname] * len(ftime))
             timeslices = np.array(timeslices)
             time = np.concatenate(timeslices)
             dataFiles = np.concatenate(np.array(dataFiles))
-            if isinstance(time[0], np.datetime64):
-                time_origin = time[0]
-                time = (time - time_origin) / np.timedelta64(1, 's')
-            else:
-                time_origin = None
-            assert(np.all((time[1:]-time[:-1]) > 0))
-
             if time.size == 1 and time[0] is None:
                 time[0] = 0
+            time_origin = TimeConverter(time[0])
+            time = time_origin.reltime(time)
+            assert(np.all((time[1:]-time[:-1]) > 0))
+
             if len(lon.shape) == 1:
                 if len(depth.shape) == 1:
                     grid = RectilinearZGrid(lon, lat, depth, time, time_origin=time_origin, mesh=mesh)
@@ -206,8 +234,8 @@ class Field(object):
             # Pre-allocate data before reading files into buffer
             data = np.empty((grid.tdim, grid.zdim, grid.ydim, grid.xdim), dtype=np.float32)
             ti = 0
-            for tslice, fname in zip(grid.timeslices, filenames):
-                with NetcdfFileBuffer(fname, dimensions, indices) as filebuffer:
+            for tslice, fname in zip(grid.timeslices, data_filenames):
+                with NetcdfFileBuffer(fname, dimensions, indices, netcdf_engine) as filebuffer:
                     # If Field.from_netcdf is called directly, it may not have a 'data' dimension
                     # In that case, assume that 'name' is the data dimension
                     filebuffer.name = filebuffer.parse_name(dimensions, variable)
@@ -233,6 +261,7 @@ class Field(object):
         kwargs['dimensions'] = dimensions.copy()
         kwargs['indices'] = indices
         kwargs['time_periodic'] = time_periodic
+        kwargs['netcdf_engine'] = netcdf_engine
 
         return cls(variable, data, grid=grid,
                    allow_time_extrapolation=allow_time_extrapolation, **kwargs)
@@ -610,10 +639,15 @@ class Field(object):
 
         :rtype: Linearly interpolated field"""
         t0 = self.grid.time[ti]
-        t1 = self.grid.time[ti+1]
-        f0 = self.data[ti, :]
-        f1 = self.data[ti+1, :]
-        return f0 + (f1 - f0) * ((time - t0) / (t1 - t0))
+        if time == t0:
+            return self.data[ti, :]
+        elif ti+1 >= len(self.grid.time):
+            raise TimeExtrapolationError(time, field=self, msg='show_time')
+        else:
+            t1 = self.grid.time[ti+1]
+            f0 = self.data[ti, :]
+            f1 = self.data[ti+1, :]
+            return f0 + (f1 - f0) * ((time - t0) / (t1 - t0))
 
     def spatial_interpolation(self, ti, z, y, x, time):
         """Interpolate horizontal field values using a SciPy interpolator"""
@@ -805,7 +839,7 @@ class Field(object):
         else:
             raise NotImplementedError('Field.write only implemented for RectilinearZGrid and CurvilinearZGrid')
 
-        attrs = {'units': 'seconds since ' + str(self.grid.time_origin)} if self.grid.time_origin else {}
+        attrs = {'units': 'seconds since ' + str(self.grid.time_origin)} if self.grid.time_origin.calendar else {}
         time_counter = xr.DataArray(self.grid.time,
                                     dims=['time_counter'],
                                     attrs=attrs)
@@ -829,12 +863,10 @@ class Field(object):
 
     def computeTimeChunk(self, data, tindex):
         g = self.grid
-        with NetcdfFileBuffer(self.dataFiles[g.ti+tindex], self.dimensions, self.indices) as filebuffer:
+        with NetcdfFileBuffer(self.dataFiles[g.ti+tindex], self.dimensions, self.indices, self.netcdf_engine) as filebuffer:
             filebuffer.name = filebuffer.parse_name(self.dimensions, self.name)
             time_data = filebuffer.time
-            if isinstance(time_data[0], np.datetime64):
-                assert isinstance(time_data[0], type(g.time_origin)), ('Field %s stores times as dates, but time_origin is not defined ' % self.name)
-                time_data = (time_data - g.time_origin) / np.timedelta64(1, 's')
+            time_data = g.time_origin.reltime(time_data)
             ti = (time_data <= g.time[tindex]).argmin() - 1
             if len(filebuffer.dataset[filebuffer.name].shape) == 2:
                 data[tindex, 0, :, :] = filebuffer.data[:, :]
@@ -904,7 +936,6 @@ class VectorField(object):
             assert self.U.grid is self.V.grid
             if W:
                 assert self.W.interp_method == 'cgrid_linear'
-                assert self.U.grid is self.W.grid
 
     def dist(self, lon1, lon2, lat1, lat2, mesh):
         if mesh == 'spherical':
@@ -1104,18 +1135,21 @@ class SummedVectorField(list):
 class NetcdfFileBuffer(object):
     """ Class that encapsulates and manages deferred access to file data. """
 
-    def __init__(self, filename, dimensions, indices):
+    def __init__(self, filename, dimensions, indices, netcdf_engine):
         self.filename = filename
         self.dimensions = dimensions  # Dict with dimension keyes for file data
         self.indices = indices
         self.dataset = None
+        self.netcdf_engine = netcdf_engine
 
     def __enter__(self):
         try:
-            self.dataset = xr.open_dataset(str(self.filename), decode_cf=True)
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=True, engine=self.netcdf_engine)
             self.dataset['decoded'] = True
         except:
-            self.dataset = xr.open_dataset(str(self.filename), decode_cf=False)
+            logger.warning_once("File %s could not be decoded properly by xarray (version %s).\n         It will be opened with no decoding. Filling values might be wrongly parsed."
+                                % (self.filename, xr.__version__))
+            self.dataset = xr.open_dataset(str(self.filename), decode_cf=False, engine=self.netcdf_engine)
             self.dataset['decoded'] = False
         for inds in self.indices.values():
             if type(inds) not in [list, range]:
@@ -1138,8 +1172,8 @@ class NetcdfFileBuffer(object):
 
     @property
     def read_lonlat(self):
-        lon = getattr(self.dataset, self.dimensions['lon'])
-        lat = getattr(self.dataset, self.dimensions['lat'])
+        lon = self.dataset[self.dimensions['lon']]
+        lat = self.dataset[self.dimensions['lat']]
         xdim = lon.size if len(lon.shape) == 1 else lon.shape[-1]
         ydim = lat.size if len(lat.shape) == 1 else lat.shape[-2]
         self.indices['lon'] = self.indices['lon'] if 'lon' in self.indices else range(xdim)
@@ -1167,7 +1201,7 @@ class NetcdfFileBuffer(object):
     @property
     def read_depth(self):
         if 'depth' in self.dimensions:
-            depth = getattr(self.dataset, self.dimensions['depth'])
+            depth = self.dataset[self.dimensions['depth']]
             depthsize = depth.size if len(depth.shape) == 1 else depth.shape[-3]
             self.indices['depth'] = self.indices['depth'] if 'depth' in self.indices else range(depthsize)
             if len(depth.shape) == 1:
@@ -1183,7 +1217,7 @@ class NetcdfFileBuffer(object):
 
     @property
     def data(self):
-        data = getattr(self.dataset, self.name)
+        data = self.dataset[self.name]
         if len(data.shape) == 2:
             data = data[self.indices['lat'], self.indices['lon']]
         elif len(data.shape) == 3:
@@ -1201,15 +1235,16 @@ class NetcdfFileBuffer(object):
     @property
     def time(self):
         try:
-            time_da = getattr(self.dataset, self.dimensions['time'])
+            time_da = self.dataset[self.dimensions['time']]
             if self.dataset['decoded'] and 'Unit' not in time_da.attrs:
-                time = np.array(time_da)
+                time = np.array([time_da]) if len(time_da.shape) == 0 else np.array(time_da)
             else:
                 if 'units' not in time_da.attrs and 'Unit' in time_da.attrs:
                     time_da.attrs['units'] = time_da.attrs['Unit']
                 ds = xr.Dataset({self.dimensions['time']: time_da})
                 ds = xr.decode_cf(ds)
-                time = np.array(getattr(ds, self.dimensions['time']))
+                da = ds[self.dimensions['time']]
+                time = np.array([da]) if len(da.shape) == 0 else np.array(da)
             if isinstance(time[0], datetime.datetime):
                 raise NotImplementedError('Parcels currently only parses dates ranging from 1678 AD to 2262 AD, which are stored by xarray as np.datetime64. If you need a wider date range, please open an Issue on the parcels github page.')
             return time
